#include <GL/gl.h>
#include <SDL/SDL.h>
#include <iostream>
#include <cmath>
#include <vector>
using namespace std;


// Classes
class vector3
{
public:
	vector3(){}
	vector3(double x, double y) : x(x), y(y), z(0) {}
	vector3(double x, double y, double z) : x(x), y(y), z(z) {}
	vector3 operator+(vector3 v){ return vector3(x+v.x, y+v.y, z+v.z); }
	vector3 operator-(vector3 v){ return vector3(x-v.x, y-v.y, z-v.z); }
	vector3 operator*(double s){ return vector3(s*x, s*y, s*z); }
	bool operator!=(vector3 v){ return (x!=v.x or y!=v.y or z!=v.z); }	
	bool operator==(vector3 v){ return (x==v.x and y==v.y and z==v.z); }
public:
	double x,y,z;
};
vector3 operator-(vector3 v) {return vector3(0,0,0) - v;}
vector3 operator/(vector3 v, double s) {return v*(1.0/s);}
vector3 operator*(double s, vector3 v) {return v*s;}
double dot(vector3 a, vector3 b) { return (a.x*b.x + a.y*b.y + a.z*b.z); }
vector3 normal(vector3 v) { return (1.0/sqrt(dot(v,v)))*v; }
template<class T> T square(T a) { return a*a; }



//Some defines
const int SCREEN_WIDTH = 400;
const int SCREEN_HEIGHT = 300;

const int XMIN = -SCREEN_WIDTH/2;
const int XMAX = SCREEN_WIDTH/2;
const int YMIN = -SCREEN_HEIGHT/2;
const int YMAX = SCREEN_HEIGHT/2;



//Plots a point
void plot(int x, int y, float r, float g, float b)
{
	glBegin(GL_POINTS);
		glColor3f(r,g,b);
		glVertex2i(x,y);
	glEnd();
}



//Main Function
int main(int nArgs, char** args)
{
	//Initialize the window
	SDL_Init(SDL_INIT_VIDEO);
	SDL_InitSubSystem(SDL_INIT_VIDEO);

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8); //8 bits for red
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8); //8 bits for green
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8); //8 bits for blue
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1); //enable page flipping

	SDL_SetVideoMode(
						SCREEN_WIDTH, SCREEN_HEIGHT, 32,
						SDL_OPENGL
					);

	SDL_WM_SetCaption("CS130 Lab", NULL);

	//Set up the projection
	glMatrixMode(GL_PROJECTION);
	glOrtho(XMIN, XMAX, YMIN, YMAX, 0, 1);
	glMatrixMode(GL_MODELVIEW);



	//Main loop
	vector3 spherePosition(0,0,10); //these variables might help
	double sphereRadius = 5;

	while(true)
	{
		//Update platform specific stuff
		SDL_Event event;
		SDL_PollEvent(&event); //Get events
		if(event.type == SDL_QUIT) //if you click the [X] exit out
			break;




		//Graphics
		glClear(GL_COLOR_BUFFER_BIT); //Clear the screen

		for(int x = XMIN; x <= XMAX; ++x)
		for(int y = YMIN; y <= YMAX; ++y)
		{
			float r = 0;
			float g = 0;
			float b = 0;
			// Do some calculation here
			// these might help
			// http://www.csee.umbc.edu/~olano/435f02/ray-sphere.html
			// http://wiki.cgsociety.org/index.php/Ray_Sphere_Intersection
			// http://www.groovyvis.com/other/raytracing/basic.html
			plot(x,y, r,g,b);
		}

		SDL_GL_SwapBuffers(); //Finished drawing
	}



	//Exit out
	SDL_Quit();
	return 0;
}
